/*
* Letter.fx
 *
 * Created on 2009-3-16, 10:17:37
 */

package loveletter;

import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.image.ImageView;
import javafx.scene.image.Image;
import javafx.scene.Node;
import javafx.animation.transition.ScaleTransition;
import javafx.animation.transition.FadeTransition;
import javafx.animation.transition.TranslateTransition;
import javafx.animation.Timeline;
import javafx.scene.input.MouseEvent;
import javafx.scene.Group;
import javafx.animation.Interpolator;
import javafx.ext.swing.SwingButton;
import javafx.scene.layout.HBox;
import javafx.scene.transform.Scale;
import javafx.animation.KeyFrame;
import javafx.scene.effect.PerspectiveTransform;
import javafx.animation.transition.PathTransition;
import javafx.animation.transition.AnimationPath;
import javafx.scene.shape.Path;
import javafx.scene.shape.MoveTo;
import javafx.scene.shape.QuadCurveTo;

/**
 * @author Joey
 */

def fwidth = 600;
def fheight = 400;

def smurfw = 256;
def smurfh = 256;

def heartw = 48;
def hearth = 48;

var beatingRate = 0.6s;

def smurf: Node = ImageView {
    x: (fwidth) / 2,
    y: (fheight - smurfh) / 2
    image: Image {
        url: "{__DIR__}smurf.png"
    }
    onMouseClicked: function( e: MouseEvent ):Void {
        beating.play();
    }
}

def smurfette: Node = ImageView {
    x: 10,
    y: 10
    image: Image {
        url: "{__DIR__}smurfette.png"
    }
    onMouseClicked: function( e: MouseEvent ):Void {
        sendRose();
    }
    effect: PerspectiveTransform {
        llx: 10 + 256,
        lly: 10 + 256
        lrx: 10,
        lry: 10 + 256
        ulx: 10 + 256,
        uly: 10
        urx: 10.0,
        ury: 10.0
    }
}

def heart: Node = ImageView {
    x: (fwidth + smurfw) / 2,
    y: fheight / 2 + smurfh / 4
    opacity: 1.0
    image: Image {
        url: "{__DIR__}heart.png"
    }
    onMouseClicked: function( e: MouseEvent ):Void {
        fallInLove();
    }
}

def heartarrow: Node = ImageView {
    x: (fwidth + smurfw) / 2,
    y: fheight / 2 + smurfh / 4
    opacity: 0.0
    image: Image {
        url: "{__DIR__}heartarrow.png"
    }
}

def heartbroken: Node = ImageView {
    x: (fwidth + smurfw) / 2,
    y: fheight / 2 + smurfh / 4
    opacity: 0.0
    image: Image {
        url: "{__DIR__}heartbroken.png"
    }
}

def heartflying = ImageView {
    transforms: [
        Scale {
            x: 1.2,
            y: 1.2
        }
    ]
    x: (fwidth + smurfw) / 2,
    y: fheight / 2 + smurfh / 4
    opacity: 0.0
    image: Image {
        url: "{__DIR__}heartflying.png"
    }
}

def hearts: Node = Group {
    content: [
        heart,
        heartarrow,
        heartbroken,
    ]
}

def avator: Node = Group {
    content: [
        smurf,
        hearts
    ]
}


def rose: Node = ImageView {
    image: Image {
        url: "{__DIR__}rose.png"
    }
    opacity: 0.0
}

var lipsScaleX = 1.0;
var lipsScaleY = 1.0;

def lips: Node = ImageView {
    transforms: [
        Scale {
            pivotX: 150 + 48 / 2,
            pivotY: 128 + 48 / 2
            x: bind lipsScaleX,
            y: bind lipsScaleY
        }
    ]
    x: 165,
    y: 128
    image: Image {
        url: "{__DIR__}kiss.png"
    }
}

def yesButton: Node = SwingButton {
    text: "Yes"
    width: 70
    action: function() {
        accept();
    }
}

def noButton: Node = SwingButton {
    text: "No"
    width: 70
    action: function() {
        reject();
    }
}

def buttons: Node = HBox {
    translateX: 70
    translateY: 270
    spacing: 10
    content: [
        yesButton, noButton
    ]
    opacity: 0.0
    disable: true
}

var beating = ScaleTransition {
    byX: 1.1,
    byY: 1.1
    node: hearts
    duration: bind beatingRate
    repeatCount: Timeline.INDEFINITE
}

var kissing = Timeline {
    keyFrames: [
        KeyFrame {
            time: 0.6s
            values: [
                lipsScaleX => 1.6 tween Interpolator.EASEIN,
                lipsScaleY => 1.6 tween Interpolator.EASEIN
            ]
        },
        KeyFrame {
            time: 0.7s
            values: [
                lipsScaleX => 1.0 tween Interpolator.LINEAR,
                lipsScaleY => 1.0 tween Interpolator.LINEAR
            ]
        }
    ]
}

def path = Path {
    elements: [
        MoveTo {
            x: heartflying.x + 48 / 2,
            y: heartflying.y + 48 / 2
        },
        QuadCurveTo {
            controlX: 600,
            controlY: 300
            x: 550,
            y: 200
        },
        QuadCurveTo {
            controlX: 450,
            controlY: 0
            x: fwidth / 2 - 140,
            y: 200
        },
    ]
}

var flying = PathTransition {
    duration: 5s
    node: heartflying
    path: AnimationPath.createFromPath(path)
}

function fadeIn(node: Node) {
    FadeTransition {
        toValue: 1.0
        duration: 1.0s
        node: node
    }.play();
}

function fadeOut(node: Node) {
    FadeTransition {
        toValue: 0.0
        duration: 1.0s
        node: node
    }.play();
}

public function fallInLove() {
    fadeOut(heart);
    fadeOut(heartbroken);
    fadeIn(heartarrow);
}

public function heartBroken() {
    fadeOut(heart);
    fadeOut(heartarrow);
    fadeIn(heartbroken);
}

public function heartFly() {
    fadeOut(heart);
    fadeOut(heartarrow);
    fadeOut(heartbroken);
    beating.stop();
    fadeIn(heartflying);
    flying.playFromStart();
}

public function sendRose() {
    fadeIn(rose);
    TranslateTransition {
        fromX: 320,
        fromY: 230
        toX: 80,
        toY: 50
        node: rose
        duration: 3s
        interpolator: Interpolator.EASEBOTH
    }.play();
}

function kiss() {
    kissing.playFromStart();
}

public function willYouMarryMe() {
    buttons.disable = false;
    fadeIn(buttons);
}

function accept() {
    buttons.disable = true;
    fadeOut(buttons);
    heartFly();
    //TODO: call javascript method sayYes()
}

function reject() {
    buttons.disable = true;
    fadeOut(buttons);
    heartBroken();
    //TODO: call javascript method sayNo()
}

var toGetHer = TranslateTransition {
    node: avator
    duration: 3s
    toX: - 150
    toY: - (fheight - smurfh) / 2 + 10
    action: function() {
        kiss();
    }
}


function run(__ARGS__: String[]) {
    Stage {
        title: "A Letter from an Unknown Geek"
        width: fwidth,
        height: fheight
        scene: Scene {
            width: fwidth,
            height: fheight
            content: [
                avator,
                smurfette,
                lips,
                rose,
                buttons,
                heartflying
            ]
        }
    }
}